@@ -4,9 +4,6 @@ import vengeance as ven
 import vengeance as vgc
 """
 from .conditional import loads_excel_module
-from .conditional import read_config_file
-
-read_config_file()
 
 from .version import *
 from .util    import *
@@ -16,10 +13,4 @@ if loads_excel_module:
     from .excel_com import *
 
 del loads_excel_module
-del read_config_file
-
-
-
-
-
 
