@@ -26,15 +26,14 @@ ordereddict:
     using about half the memory of collections.OrderedDict 
 
 '''
-config          = {}
-__config_already_read__ = False
+config             = {}
+config_file_loaded = False
 
 python_version      = sys.version_info
-is_windows_os       = (os.name == 'nt') or (sys.platform == 'win32')
+is_windows_os       = (os.name == 'nt' or sys.platform == 'win32')
 is_utf_console      = ('utf' in sys.stdout.encoding.lower())
-is_tty_console      = False
-is_pypy_interpreter = ('__pypy__' in iter(sys.builtin_module_names))
-loads_excel_module  = (not is_pypy_interpreter) and is_windows_os
+is_pypy_interpreter = ('__pypy__' in sys.builtin_module_names)
+loads_excel_module  = (not is_pypy_interpreter and is_windows_os)
 
 ordereddict             = dict
 dateutil_installed      = False
@@ -73,50 +72,18 @@ except ImportError:
     pass
 
 
-def enable_ansi_escape_in_console():
-    """ attempt to turn on ansi escape colors in console
-
-    ansi escape: \x1b, \033, <0x1b>, 
-    how to effectively check if console supports ansi escapes?
-        ansi escape shows up as '←' in python.exe console
-
-    STD_OUTPUT_HANDLE = -11
-    DWMODE            = 7
-    """
-    global is_utf_console
-    global is_tty_console
-
-    if not is_windows_os:
-        return
-
-    try:
-        import ctypes
-        kernel32 = ctypes.windll.kernel32
-        kernel32.SetConsoleMode(kernel32.GetStdHandle(-11), 7)
-
-        is_utf_console = True
-        is_tty_console = False
-    except:
-        pass
-
-
-def read_config_file():
+def load_vengeance_configuration_file():
     global config
-    global __config_already_read__
+    global config_file_loaded
 
-    from configparser import ConfigParser
+    config_file_loaded = True
 
-    __config_already_read__ = True
-
-    config_path = __locate_config_path()
-    if config_path is None:
+    vcf = __load_vengeance_configuration_file()
+    if vcf is None:
         return
 
-    cp = ConfigParser()
-    cp.read(config_path)
-
-    if cp.has_section('console'):
-        cp_section = cp['console']
+    if vcf.has_section('console'):
+        cp_section = vcf['console']
 
         config.update({'color':              cp_section.get('color'),
                        'effect':             cp_section.get('effect'),
@@ -124,32 +91,79 @@ def read_config_file():
                        'enable_ansi_escape': cp_section.getboolean('enable_ansi_escape')})
 
         if config['enable_ansi_escape']:
-            enable_ansi_escape_in_console()
+            __enable_ansi_escape_in_windows_console()
+
+    # sys.getfilesystemencoding()
 
     # if cp.has_section('filesystem'):
     #     cp_section = cp['filesystem']
     #     config.update({'encoding': cp_section.get('encoding')})
 
 
-def __locate_config_path():
+def __load_vengeance_configuration_file():
     import site
 
-    config_paths = [site.getsitepackages()[1]  + '\\vengeance\\config.ini',
-                    os.environ['localappdata'] + '\\Temp\\vengeance\\config.ini']
-    if not is_windows_os:
-        del config_paths[-1]
+    config_paths = [site.getsitepackages()[1]  + '\\vengeance\\config.ini']
+
+    if is_windows_os:
+        try:    config_paths.append(os.environ['localappdata'] + '\\Temp\\vengeance\\config.ini')
+        except: pass
 
     for config_path in config_paths:
         if os.path.exists(config_path):
-            return config_path
+
+            from configparser import ConfigParser
+
+            vcf = ConfigParser()
+            vcf.read(config_path)
+
+            return vcf
 
     return None
 
 
-if __config_already_read__ is False:
-    read_config_file()
+def __enable_ansi_escape_in_windows_console():
+    """ attempt to turn on ansi escape colors in console
+
+    ansi escape: \x1b, \033, <0x1b>, 
+    how to effectively check if console supports ansi escapes?
+        ansi escape shows up as '←' in python.exe console
+
+    # os.popen('chcp 65001')
+
+    import codecs
+    import locale
+    a = codecs.getwriter(locale.getpreferredencoding())(sys.stdout)
+
+    STD_OUTPUT_HANDLE = -11
+    DWMODE            = 7
+
+    # GetConsoleMode = ctypes.windll.kernel32.GetConsoleMode
+    # print(f'{GetConsoleMode(STD_OUTPUT_HANDLE) = }')
+    """
+    if not is_windows_os:
+        return
+
+    try:
+        import ctypes
+        global is_utf_console
+
+        SetConsoleMode    = ctypes.windll.kernel32.SetConsoleMode
+        GetStdHandle      = ctypes.windll.kernel32.GetStdHandle
+        STD_OUTPUT_HANDLE = -11
+        DWMODE            = 7
+
+        SetConsoleMode(GetStdHandle(STD_OUTPUT_HANDLE), DWMODE)
+        is_utf_console = True
+
+    except:
+        is_utf_console = False
+
+
+if config_file_loaded is False:
+    load_vengeance_configuration_file()
+
 
-# is_tty_console          = (hasattr(sys.stdout, 'isatty') and sys.stdout.isatty())
 # is_pypy_interpreter     = True
 # is_windows_os           = True
 # dateutil_installed      = False
